@@ -1,4 +1,4 @@
-# Copyright (c) 2007 by Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
+# Copyright (c) 2007-2008 by Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
 #
 # This file is part of PyCha.
 #
@@ -16,6 +16,8 @@
 # along with PyCha.  If not, see <http://www.gnu.org/licenses/>.
 
 import copy
+import math
+
 import cairo
 
 from pycha.color import (defaultColorscheme, getColorscheme, hex2rgb,
@@ -41,7 +43,7 @@ class Chart(object):
         self.yscale = 1.0
         self.xrange = None
         self.yrange = None
-        
+
         self.xticks = []
         self.yticks = []
 
@@ -80,18 +82,19 @@ class Chart(object):
 
     def render(self, surface=None, options={}):
         """Renders the chart with the specified options.
-        
+
         The optional parameters can be used to render a chart in a different
         surface with new options.
         """
         self._update(options)
         if surface:
             self._initSurface(surface)
-        
+
         cx = cairo.Context(self.surface)
         self._renderBackground(cx)
         self._renderChart(cx)
         self._renderAxis(cx)
+        self._renderTitle(cx)
         self._renderLegend(cx)
 
     def clean(self):
@@ -122,7 +125,7 @@ class Chart(object):
         if self.resetFlag:
             self.resetFlag = False
             self.clean()
-        
+
 
     # update methods
     def _update(self, options={}):
@@ -135,59 +138,65 @@ class Chart(object):
 
     def _updateXY(self):
         """Calculates all kinds of metrics for the x and y axis"""
-        stores = self._getDatasetsValues()
+        x_range_is_defined = self.options.axis.x.range is not None
+        y_range_is_defined = self.options.axis.y.range is not None
+
+        if not x_range_is_defined or not y_range_is_defined:
+            stores = self._getDatasetsValues()
+
+        # gather data for the x axis
+        if x_range_is_defined:
+            self.minxval, self.maxxval = self.options.axis.x.range
+        else:
+            xdata = [pair[0] for pair in reduce(lambda a,b: a+b, stores)]
+            self.minxval = float(min(xdata))
+            self.maxxval = float(max(xdata))
+            if self.minxval * self.maxxval > 0 and self.minxval > 0:
+                self.minxval = 0.0
+
+        self.xrange = self.maxxval - self.minxval
+        if self.xrange == 0:
+            self.xscale = 1.0
+        else:
+            self.xscale = 1 / self.xrange
+
+        # gather data for the y axis
+        if y_range_is_defined:
+            self.minyval, self.maxyval = self.options.axis.y.range
+        else:
+            ydata = [pair[1] for pair in reduce(lambda a,b: a+b, stores)]
+            self.minyval = float(min(ydata))
+            self.maxyval = float(max(ydata))
+            if self.minyval * self.maxyval > 0 and self.minyval > 0:
+                self.minyval = 0.0
+
+        self.yrange = self.maxyval - self.minyval
+        if self.yrange == 0:
+            self.yscale = 1.0
+        else:
+            self.yscale = 1 / self.yrange
 
         # calculate area data
         width = (self.surface.get_width()
                  - self.options.padding.left - self.options.padding.right)
         height = (self.surface.get_height()
                   - self.options.padding.top - self.options.padding.bottom)
+
+        if self.minyval * self.maxyval < 0: # different signs
+            origin = abs(self.minyval) * self.yscale
+        else:
+            origin = 0
+
         self.area = Area(self.options.padding.left,
                          self.options.padding.top,
-                         width, height)
-
-        # gather data for the x axis
-        if self.options.axis.x.range:
-            self.minxval, self.maxxval = self.options.axis.x.range
-            self.xscale = self.maxxval - self.minxval
-        else:
-            xdata = [pair[0] for pair in reduce(lambda a,b: a+b, stores)]
-            if self.options.xOriginIsZero:
-                self.minxval = 0.0 
-            else:
-                self.minxval = float(min(xdata))
-            self.maxxval = float(max(xdata))
-
-        self.xrange = self.maxxval - self.minxval        
-        if self.xrange == 0:
-            self.xscale = 1.0 
-        else:
-            self.xscale = 1 / self.xrange
-
-        # gather data for the y axis
-        if self.options.axis.y.range:
-            self.minyval, self.maxyval = self.options.axis.y.range
-            self.yscale = self.maxyval - self.minyval
-        else:
-            ydata = [pair[1] for pair in reduce(lambda a,b: a+b, stores)]
-            if self.options.yOriginIsZero:
-                self.minyval = 0.0  
-            else:
-                self.minyval = float(min(ydata))
-            self.maxyval = float(max(ydata))
-
-        self.yrange = self.maxyval - self.minyval
-        if self.yrange == 0:
-            self.yscale = 1.0 
-        else:
-            self.yscale = 1 / self.yrange
+                         width, height, origin)
 
     def _updateChart(self):
         raise NotImplementedError
 
     def _updateTicks(self):
         """Evaluates ticks for x and y axis.
-        
+
         You should call _updateXY before because that method computes the
         values of xscale, minxval, yscale, and other attributes needed for
         this method.
@@ -202,7 +211,7 @@ class Chart(object):
                     tick = Option(tick)
                 if tick.label is None:
                     label = str(tick.v)
-                else: 
+                else:
                     label = tick.label
                 pos = self.xscale * (tick.v - self.minxval)
                 if 0.0 <= pos <= 1.0:
@@ -211,27 +220,26 @@ class Chart(object):
         elif self.options.axis.x.tickCount > 0:
             uniqx = range(len(uniqueIndices(stores)) + 1)
             roughSeparation = self.xrange / self.options.axis.x.tickCount
-
             i = j = 0
-            while i + 1 < len(uniqx) and j < self.options.axis.x.tickCount:
-                if (uniqx[i + 1] - self.minxval) >= (j * roughSeparation):
+            while i  < len(uniqx) and j < self.options.axis.x.tickCount:
+                if (uniqx[i] - self.minxval) >= (j * roughSeparation):
                     pos = self.xscale * (uniqx[i] - self.minxval)
                     if 0.0 <= pos <= 1.0:
-                        self.xticks.append((pos, uniqx[i + 1]))
+                        self.xticks.append((pos, uniqx[i]))
                         j += 1
                 i += 1
 
         # evaluate yTicks
         self.yticks = []
         if self.options.axis.y.ticks:
-            for tick in self.options.y.ticks:
+            for tick in self.options.axis.y.ticks:
                 if not isinstance(tick, Option):
                     tick = Option(tick)
                 if tick.label is None:
-                    label = str(tick.v)  
+                    label = str(tick.v)
                 else:
                     label = tick.label
-                pos = self.yscale * (tick.v - self.minyval)
+                pos = 1.0 - (self.yscale * (tick.v - self.minyval))
                 if 0.0 <= pos <= 1.0:
                     self.yticks.append((pos, label))
 
@@ -242,28 +250,35 @@ class Chart(object):
                 roughSeparation = 1
             else:
                 roughSeparation = round(num, prec)
-            
+
             for i in range(self.options.axis.y.tickCount + 1):
                 yval = self.minyval + (i * roughSeparation)
                 pos = 1.0 - ((yval - self.minyval) * self.yscale)
                 if 0.0 <= pos <= 1.0:
                     self.yticks.append((pos, round(yval, prec)))
-            
+
     # render methods
     def _renderBackground(self, cx):
         """Renders the background area of the chart"""
         if self.options.background.hide:
             return
-        
+
         cx.save()
-        cx.set_source_rgb(*hex2rgb(self.options.background.color))
-        cx.rectangle(self.area.x, self.area.y, self.area.w, self.area.h)
-        cx.fill()
-        cx.set_source_rgb(*hex2rgb(self.options.background.lineColor))
-        cx.set_line_width(self.options.axis.lineWidth)
-        
-        self._renderLines(cx)
-        
+
+        if self.options.background.baseColor:
+            cx.set_source_rgb(*hex2rgb(self.options.background.baseColor))
+            cx.paint()
+
+        if self.options.background.chartColor:
+            cx.set_source_rgb(*hex2rgb(self.options.background.chartColor))
+            cx.rectangle(self.area.x, self.area.y, self.area.w, self.area.h)
+            cx.fill()
+
+        if self.options.background.lineColor:
+            cx.set_source_rgb(*hex2rgb(self.options.background.lineColor))
+            cx.set_line_width(self.options.axis.lineWidth)
+            self._renderLines(cx)
+
         cx.restore()
 
     def _renderLines(self, cx):
@@ -271,9 +286,9 @@ class Chart(object):
         ticks = self.yticks
         for tick in ticks:
             self._renderLine(cx, tick, False)
-        
+
     def _renderLine(self, cx, tick, horiz):
-        """Aux function for _renderLines""" 
+        """Aux function for _renderLines"""
         x1, x2, y1, y2 = (0, 0, 0, 0)
         if horiz:
             x1 = x2 = tick[0] * self.area.w + self.area.x
@@ -293,87 +308,208 @@ class Chart(object):
     def _renderChart(self, cx):
         raise NotImplementedError
 
+    def _renderYTick(self, cx, tick):
+        """Aux method for _renderAxis"""
+
+        if callable(tick):
+            return
+
+        x = self.area.x
+        y = self.area.y + tick[0] * self.area.h
+
+        cx.new_path()
+        cx.move_to(x, y)
+        cx.line_to(x - self.options.axis.tickSize, y)
+        cx.close_path()
+        cx.stroke()
+
+        label =  unicode(tick[1])
+        extents = cx.text_extents(label)
+        labelWidth = extents[2]
+        labelHeight = extents[3]
+
+        if self.options.axis.y.rotate:
+            radians = math.radians(self.options.axis.y.rotate)
+            cx.move_to(x - self.options.axis.tickSize
+                       - (labelWidth * math.cos(radians))
+                       - 4,
+                       y + (labelWidth * math.sin(radians))
+                       + labelHeight / (2.0 / math.cos(radians)))
+            cx.rotate(-radians)
+            cx.show_text(label)
+            cx.rotate(radians) # this is probably faster than a save/restore
+        else:
+            cx.move_to(x - self.options.axis.tickSize - labelWidth - 4,
+                       y + labelHeight / 2.0)
+            cx.show_text(label)
+
+        return label
+
+    def _renderXTick(self, cx, tick, fontAscent):
+        if callable(tick):
+            return
+
+        x = self.area.x + tick[0] * self.area.w
+        y = self.area.y + self.area.h
+
+        cx.new_path()
+        cx.move_to(x, y)
+        cx.line_to(x, y + self.options.axis.tickSize)
+        cx.close_path()
+        cx.stroke()
+
+        label = unicode(tick[1])
+        extents = cx.text_extents(label)
+        labelWidth = extents[2]
+        labelHeight = extents[3]
+
+        if self.options.axis.x.rotate:
+            radians = math.radians(self.options.axis.x.rotate)
+            cx.move_to(x - (labelHeight * math.cos(radians)),
+                       y + self.options.axis.tickSize
+                       + (labelHeight * math.cos(radians))
+                       + 4.0)
+            cx.rotate(radians)
+            cx.show_text(label)
+            cx.rotate(-radians)
+        else:
+            cx.move_to(x - labelWidth / 2.0,
+                       y + self.options.axis.tickSize
+                       + fontAscent + 4.0)
+            cx.show_text(label)
+        return label
+
+    def _getTickSize(self, cx, ticks, rotate):
+        tickExtents = [cx.text_extents(unicode(tick[1]))[2:4] for tick in ticks]
+        tickWidth = tickHeight = 0.0
+        if tickExtents:
+            tickHeight = self.options.axis.tickSize + 4.0
+            tickWidth = self.options.axis.tickSize + 4.0
+            widths, heights = zip(*tickExtents)
+            maxWidth, maxHeight = max(widths), max(heights)
+            if rotate:
+                radians = math.radians(rotate)
+                sinRadians = math.sin(radians)
+                cosRadians = math.cos(radians)
+                maxHeight = maxWidth * sinRadians + maxHeight * cosRadians
+                maxWidth =  maxWidth * cosRadians + maxHeight * sinRadians
+            tickWidth += maxWidth
+            tickHeight += maxHeight
+        return tickWidth, tickHeight
+
+    def _renderAxisLabel(self, cx, tickWidth, tickHeight, label, x, y, vertical=False):
+        cx.new_path()
+        cx.select_font_face(self.options.axis.labelFont,
+                            cairo.FONT_SLANT_NORMAL,
+                            cairo.FONT_WEIGHT_BOLD)
+        labelWidth = cx.text_extents(label)[2]
+        fontAscent = cx.font_extents()[0]
+        if vertical:
+            cx.move_to(x, y + labelWidth / 2)
+            radians = math.radians(90)
+            cx.rotate(-radians)
+        else:
+            cx.move_to(x - labelWidth / 2.0, y + fontAscent)
+
+        cx.show_text(label)
+
+    def _renderYAxis(self, cx):
+        """Draws the vertical line represeting the Y axis"""
+        cx.new_path()
+        cx.move_to(self.area.x, self.area.y)
+        cx.line_to(self.area.x, self.area.y + self.area.h)
+        cx.close_path()
+        cx.stroke()
+
+    def _renderXAxis(self, cx):
+        """Draws the horizontal line representing the X axis"""
+        cx.new_path()
+        cx.move_to(self.area.x,
+                   self.area.y + self.area.h * (1.0 - self.area.origin))
+        cx.line_to(self.area.x + self.area.w,
+                   self.area.y + self.area.h * (1.0 - self.area.origin))
+        cx.close_path()
+        cx.stroke()
+
     def _renderAxis(self, cx):
         """Renders axis"""
         if self.options.axis.x.hide and self.options.axis.y.hide:
             return
-        
+
         cx.save()
         cx.set_source_rgb(*hex2rgb(self.options.axis.lineColor))
         cx.set_line_width(self.options.axis.lineWidth)
-        
+
         if not self.options.axis.y.hide:
             if self.yticks:
-                def drawYLabel(tick):
-                    if callable(tick):
-                        return
-                    
-                    x = self.area.x
-                    y = self.area.y + tick[0] * self.area.h
-                    
-                    cx.new_path()
-                    cx.move_to(x, y)
-                    cx.line_to(x - self.options.axis.tickSize, y)
-                    cx.close_path()
-                    cx.stroke()
-                    
-                    label =  unicode(tick[1])
-                    extents = cx.text_extents(label)
-                    labelWidth = extents[2]
-                    labelHeight = extents[3]
-                    cx.move_to(x - self.options.axis.tickSize - labelWidth - 5,
-                               y + labelHeight / 2.0)
-                    cx.show_text(label)
-                    
-                    return label
                 for tick in self.yticks:
-                    drawYLabel(tick)
-                
-            cx.new_path()
-            cx.move_to(self.area.x, self.area.y)
-            cx.line_to(self.area.x, self.area.y + self.area.h)
-            cx.close_path()
-            cx.stroke()
+                    self._renderYTick(cx, tick)
+
+            if self.options.axis.y.label:
+                cx.save()
+                rotate = self.options.axis.y.rotate
+                tickWidth, tickHeight = self._getTickSize(cx, self.yticks,
+                                                          rotate)
+                label = unicode(self.options.axis.y.label)
+                x = self.area.x - tickWidth - 4.0
+                y = self.area.y + 0.5 * self.area.h
+                self._renderAxisLabel(cx, tickWidth, tickHeight, label, x, y,
+                                      True)
+                cx.restore()
+
+            self._renderYAxis(cx)
 
         if not self.options.axis.x.hide:
+            fontAscent = cx.font_extents()[0]
             if self.xticks:
-                def drawXLabel(tick):
-                    if callable(tick):
-                        return
-                    
-                    x = self.area.x + tick[0] * self.area.w
-                    y = self.area.y + self.area.h
-                    
-                    cx.new_path()
-                    cx.move_to(x, y)
-                    cx.line_to(x, y + self.options.axis.tickSize)
-                    cx.close_path()
-                    cx.stroke()
-                    
-                    label = unicode(tick[1])
-                    extents = cx.text_extents(label)
-                    labelWidth = extents[2]
-                    labelHeight = extents[3]
-                    cx.move_to(x - labelWidth / 2.0,
-                               y + self.options.axis.tickSize + 10)
-                    cx.show_text(label)
-                    return label
                 for tick in self.xticks:
-                    drawXLabel(tick)
-            
-            cx.new_path()
-            cx.move_to(self.area.x, self.area.y + self.area.h)
-            cx.line_to(self.area.x + self.area.w, self.area.y + self.area.h)
-            cx.close_path()
-            cx.stroke()
+                    self._renderXTick(cx, tick, fontAscent)
+
+            if self.options.axis.x.label:
+                cx.save()
+                rotate = self.options.axis.x.rotate
+                tickWidth, tickHeight = self._getTickSize(cx, self.xticks,
+                                                          rotate)
+                label = unicode(self.options.axis.x.label)
+                x = self.area.x + self.area.w / 2.0
+                y = self.area.y + self.area.h + tickHeight + 4.0
+                self._renderAxisLabel(cx, tickWidth, tickHeight, label, x, y,
+                                      False)
+                cx.restore()
+
+            self._renderXAxis(cx)
 
         cx.restore()
 
+    def _renderTitle(self, cx):
+        if self.options.title:
+            cx.save()
+            cx.select_font_face(self.options.titleFont,
+                                cairo.FONT_SLANT_NORMAL,
+                                cairo.FONT_WEIGHT_BOLD)
+            cx.set_font_size(self.options.titleFontSize)
+
+            title = unicode(self.options.title)
+            extents = cx.text_extents(title)
+            titleWidth = extents[2]
+
+            x = self.area.x + self.area.w / 2.0 - titleWidth / 2.0
+            y = cx.font_extents()[0] # font ascent
+
+            cx.move_to(x, y)
+            cx.show_text(title)
+
+            cx.restore()
+
     def _renderLegend(self, cx):
         """This function adds a legend to the chart"""
         if self.options.legend.hide:
             return
-        
+
+        surfaceWidth = self.surface.get_width()
+        surfaceHeight = self.surface.get_height()
+
+        # Compute legend dimensions
         padding = 4
         bullet = 15
         width = 0
@@ -385,6 +521,18 @@ class Chart(object):
             height += max(extents[3], bullet) + padding
         width = padding + bullet + padding + width + padding
 
+        # Compute legend position
+        legend = self.options.legend
+        if legend.position.right is not None:
+            legend.position.left = (surfaceWidth
+                                    - legend.position.right
+                                    - width)
+        if legend.position.bottom is not None:
+            legend.position.top = (surfaceHeight
+                                   - legend.position.bottom
+                                   - height)
+
+        # Draw the legend
         cx.save()
         cx.rectangle(self.options.legend.position.left,
                      self.options.legend.position.top,
@@ -394,7 +542,7 @@ class Chart(object):
         cx.set_line_width(self.options.stroke.width)
         cx.set_source_rgb(*hex2rgb(self.options.legend.borderColor))
         cx.stroke()
-        
+
         def drawKey(key, x, y, text_height):
             cx.rectangle(x, y, bullet, bullet)
             cx.set_source_rgb(*self.options.colorScheme[key])
@@ -404,7 +552,7 @@ class Chart(object):
             cx.move_to(x + bullet + padding,
                        y + bullet / 2.0 + text_height / 2.0)
             cx.show_text(key)
-        
+
         cx.set_line_width(1)
         x = self.options.legend.position.left + padding
         y = self.options.legend.position.top + padding
@@ -418,11 +566,16 @@ class Chart(object):
 def uniqueIndices(arr):
     """Return a list with the indexes of the biggest element of arr"""
     return range(max([len(a) for a in arr]))
-            
+
 class Area(object):
     """Simple rectangle to hold an area coordinates and dimensions"""
-    def __init__(self, x, y, w, h):
+    def __init__(self, x, y, w, h, origin=0.0):
         self.x, self.y, self.w, self.h = x, y, w, h
+        self.origin = origin
+
+    def __str__(self):
+        msg = "<pycha.chart.Area@(%.2f, %.2f) %.2f x %.2f Origin: %.2f>"
+        return  msg % (self.x, self.y, self.w, self.h, self.origin)
 
 class Option(dict):
     """Useful dict that allow attribute-like access to its keys"""
@@ -444,7 +597,7 @@ class Option(dict):
 DEFAULT_OPTIONS = Option(
     axis=Option(
         lineWidth=1.0,
-        lineColor='#000000',
+        lineColor='#0f0000',
         tickSize=3.0,
         labelColor='#666666',
         labelFont='Tahoma',
@@ -456,6 +609,8 @@ DEFAULT_OPTIONS = Option(
             tickCount=10,
             tickPrecision=1,
             range=None,
+            rotate=None,
+            label=None,
         ),
         y=Option(
             hide=False,
@@ -463,26 +618,28 @@ DEFAULT_OPTIONS = Option(
             tickCount=10,
             tickPrecision=1,
             range=None,
+            rotate=None,
+            label=None,
         ),
     ),
     background=Option(
-        color='#f5f5f5',
         hide=False,
+        baseColor=None,
+        chartColor='#f5f5f5',
         lineColor='#ffffff',
         lineWidth=1.5,
     ),
     legend=Option(
         opacity=0.8,
         borderColor='#000000',
-        style={},
         hide=False,
-        position=Option(top=20, left=40),
+        position=Option(top=20, left=40, bottom=None, right=None),
     ),
     padding=Option(
         left=30,
         right=30,
-        top=15,
-        bottom=15,
+        top=30,
+        bottom=30,
     ),
     stroke=Option(
         color='#ffffff',
@@ -493,8 +650,9 @@ DEFAULT_OPTIONS = Option(
     fillOpacity=1.0,
     shouldFill=True,
     barWidthFillFraction=0.75,
-    xOriginIsZero=True,
-    yOriginIsZero=True,
     pieRadius=0.4,
     colorScheme=DEFAULT_COLOR,
+    title=None,
+    titleFont='Tahoma',
+    titleFontSize=12,
 )
