@@ -1,4 +1,4 @@
-# Copyright (c) 2007 by Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
+# Copyright (c) 2007-2008 by Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
 #
 # This file is part of PyCha.
 #
@@ -16,14 +16,14 @@
 # along with PyCha.  If not, see <http://www.gnu.org/licenses/>.
 
 from pycha.chart import Chart
-from pycha.color import hex2rgb, clamp
+from pycha.color import hex2rgb
 
 class LineChart(Chart):
-    
+
     def __init__(self, surface=None, options={}):
         super(LineChart, self).__init__(surface, options)
         self.points = []
-        
+
     def _updateChart(self):
         """Evaluates measures for line charts"""
         self.points = []
@@ -33,29 +33,47 @@ class LineChart(Chart):
                 xval, yval = item
                 x = (xval - self.minxval) * self.xscale
                 y = 1.0 - (yval - self.minyval) * self.yscale
-                point = Point(x, clamp(0.0, 1.0, y), xval, yval, name)
-                
-                if 0.0 <= point.x <= 1.0:
+                point = Point(x, y, xval, yval, name)
+
+                if 0.0 <= point.x <= 1.0 and 0.0 <= point.y <= 1.0:
                     self.points.append(point)
-    
+
     def _renderChart(self, cx):
-        """Renders a line chart"""        
+        """Renders a line chart"""
         def preparePath(storeName):
             cx.new_path()
-            cx.move_to(self.area.x, self.area.y + self.area.h)
+            firstPoint = True
+            lastX = None
+            if self.options.shouldFill:
+                # Go to the (0,0) coordinate to start drawing the area
+                #cx.move_to(self.area.x, self.area.y + self.area.h)
+                cx.move_to(self.area.x,
+                           self.area.y + (1.0 - self.area.origin) * self.area.h)
+
             for point in self.points:
                 if point.name == storeName:
+                    if not self.options.shouldFill and firstPoint:
+                        # starts the first point of the line
+                        cx.move_to(point.x * self.area.w + self.area.x,
+                                   point.y * self.area.h + self.area.y)
+                        firstPoint = False
+                        continue
                     cx.line_to(point.x * self.area.w + self.area.x,
                                point.y * self.area.h + self.area.y)
-            cx.line_to(self.area.w + self.area.x, self.area.h + self.area.y)
-            cx.line_to(self.area.x, self.area.y + self.area.h)
+                    # we remember the last X coordinate to close the area
+                    # properly. See bug #4
+                    lastX = point.x
 
             if self.options.shouldFill:
+                # Close the path to the start point
+                y = (1.0 - self.area.origin) * self.area.h + self.area.y
+                cx.line_to(lastX * self.area.w + self.area.x, y)
+                cx.line_to(self.area.x, y)
                 cx.close_path()
             else:
                 cx.set_source_rgb(*self.options.colorScheme[storeName])
                 cx.stroke()
-        
+
 
         cx.save()
         cx.set_line_width(self.options.stroke.width)
@@ -69,12 +87,12 @@ class LineChart(Chart):
                     preparePath(storeName)
                     cx.fill()
                     cx.restore()
-                
+
                 # fill the line
                 cx.set_source_rgb(*self.options.colorScheme[storeName])
                 preparePath(storeName)
                 cx.fill()
-                
+
                 if not self.options.stroke.hide:
                     # draw stroke
                     cx.set_source_rgb(*hex2rgb(self.options.stroke.color))
@@ -94,4 +112,7 @@ class Point(object):
     def __init__(self, x, y, xval, yval, name):
         self.x, self.y = x, y
         self.xval, self.yval = xval, yval
-        self.name = name
\ No newline at end of file
+        self.name = name
+
+    def __str__(self):
+        return "<pycha.line.Point@(%.2f, %.2f)>" % (self.x, self.y)
